@@ -1,9 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+/* A Bison parser, made from /home/anholt/xserver-stable/hw/dmx/config/parser.y, by GNU bison 1.75.  */
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+/* Skeleton parser for Yacc-like parsing with Bison,
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,21 +15,16 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
-/* As a special exception, you may create a larger work that contains
-   part or all of the Bison parser skeleton and distribute that work
-   under terms of your choice, so long as that work isn't itself a
-   parser generator using the skeleton or a modified version thereof
-   as a parser skeleton.  Alternatively, if you modify or redistribute
-   the parser skeleton itself, you may (at your option) remove this
-   special exception, which will cause the skeleton and the resulting
-   Bison output files to be licensed under the GNU General Public
-   License without this special exception.
+/* As a special exception, when this file is copied by Bison into a
+   Bison output file, you may use that output file without restriction.
+   This special exception was added by the Free Software Foundation
+   in version 1.24 of Bison.  */
 
-   This special exception was added by the Free Software Foundation in
-   version 2.2 of Bison.  */
+#ifndef BISON_PARSER_H
+# define BISON_PARSER_H
 
 /* Tokens.  */
 #ifndef YYTOKENTYPE
@@ -52,7 +45,6 @@
      T_LINE_COMMENT = 268
    };
 #endif
-/* Tokens.  */
 #define T_VIRTUAL 258
 #define T_DISPLAY 259
 #define T_WALL 260
@@ -68,10 +60,9 @@
 
 
 
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
-#line 56 "../../../../hw/dmx/config/parser.y"
-{
+#ifndef YYSTYPE
+#line 56 "parser.y"
+typedef union {
     DMXConfigTokenPtr      token;
     DMXConfigStringPtr     string;
     DMXConfigNumberPtr     number;
@@ -86,14 +77,14 @@ typedef union YYSTYPE
     DMXConfigSubPtr        subentry;
     DMXConfigVirtualPtr    virtual;
     DMXConfigEntryPtr      entry;
-}
-/* Line 1529 of yacc.c.  */
-#line 92 "parser.h"
-	YYSTYPE;
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
-# define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
+} yystype;
+/* Line 1237 of /usr/local/share/bison/yacc.c.  */
+#line 83 "parser.h"
+# define YYSTYPE yystype
 #endif
 
 extern YYSTYPE yylval;
 
+
+#endif /* not BISON_PARSER_H */
+
